package com.huaxia.imes.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huaxia.comm.domain.imes.Cutpiece;
import com.huaxia.imes.dto.CutpieceDTO;
import com.huaxia.imes.mapper.CutpieceMapper;
import com.ruoyi.common.annotation.DataSource;
import com.ruoyi.common.enums.DataSourceType;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author zx
 * @since 2024年10月18日
 */
@Service
@Slf4j
@AllArgsConstructor
@DataSource(DataSourceType.SLAVE)
@Transactional(rollbackFor = Exception.class)
public class CutpieceService extends ServiceImpl<CutpieceMapper, Cutpiece> implements IService<Cutpiece> {

    private CutpieceMapper cutpieceMapper;


    /**
     * 获取生产
     *
     * @return
     */
    public List<Cutpiece> queryList(CutpieceDTO dto) {

        //获取上班时间
        LocalTime time = dto.getWorkTime();
        //获取目标产能
        double targetCapacity = dto.getTargetCapacity();
        // 获取当前日期
        LocalDate currDate = LocalDate.now();
        //减三天 todo(用来获取测试数据)
        currDate = currDate.minusDays(3);

        // 当天零点零分
        LocalDateTime startTime = currDate.atTime(LocalTime.MIN);
        //获取当天 23:59:59
        LocalDateTime endTime = currDate.atTime(LocalTime.MAX);
        if (dto.getWorkStartTime() == null) {
            dto.setWorkStartTime(startTime);
        }
        if (dto.getWorkEndTime() == null) {
            dto.setWorkEndTime(endTime);
        }
        // 查询当天数据
        List<Cutpiece> bo = cutpieceMapper.queryList(dto);
        //有数据 开始逻辑处理
        if (bo != null && bo.size() > 0) {
            for (Cutpiece cutpiece : bo) {
                //获取切割用时  时间秒 换算成小时
                Long count = cutpiece.getTotalTime();
                double totalTimeHoursWithFraction = count / 3600.0;
                DecimalFormat Format = new DecimalFormat("#.##");
                cutpiece.setRunTime(Double.valueOf(Format.format(totalTimeHoursWithFraction)));
                /*log.info("机器运行时间:{}", totalTimeHoursWithFraction);*/
                //机器上班时间 通过当前年月日 拼接 时间
                String workTimeString = currDate.format(DateTimeFormatter.ISO_LOCAL_DATE) + " " + time;
                LocalDateTime workTime = LocalDateTime.parse(workTimeString, DateTimeFormatter.ofPattern("yyyy-MM-dd H:mm"));
                cutpiece.setWorkingTime(workTime);
                /*log.info("上班时间:{}", workTime);*/
                //机器 开始运作时间
                LocalDateTime workStartTime = cutpiece.getStartTime();
                //机器 结束运作时间
                LocalDateTime workEndTime = cutpiece.getEndTime();
                //计算机器运作时间致机器上班时间的差值
                Duration duration = Duration.between(workStartTime, workEndTime);
                //差值换算成秒
                long workTimeSeconds = duration.getSeconds();
                //机器大致运作时长-机器实际运作时长=机器闲置时长
                long idleTime = workTimeSeconds - count;
                //换算成小时
                double idleTimeHoursWithFraction = idleTime / 3600.0;
                //保留小数点后两位
                DecimalFormat format = new DecimalFormat("#.##");
                cutpiece.setFreeTime(Double.valueOf(format.format(idleTimeHoursWithFraction)));
                /*log.info("机器闲置时间:{}", idleTimeHoursWithFraction);*/
                cutpiece.setTarget(targetCapacity);
                //换算百分比
                double completeRatePercentage = (cutpiece.getCount() / targetCapacity) * 100;
                cutpiece.setFinish(completeRatePercentage);
                /*log.info("完成率:{}%", completeRatePercentage);*/
            }
        }
        return bo;
    }
}
